@@ -8,7 +8,7 @@
 
 # Package name      : perl5
 # Source directory  : .
-# Configuration time: Sun Jul 25 17:08:22 EET DST 1999
+# Configuration time: Mon Aug  2 01:32:52 EET DST 1999
 # Configured by     : jhi
 # Target system     : osf1 alpha.hut.fi v4.0 878 alpha 
 
@@ -30,10 +30,10 @@ afs='false'
 alignbytes='8'
 ansi2knr=''
 aphostname=''
-apiversion='5.00557'
+apiversion='5.00558'
 ar='ar'
-archlib='/opt/perl/lib/5.00557/alpha-dec_osf-thread'
-archlibexp='/opt/perl/lib/5.00557/alpha-dec_osf-thread'
+archlib='/opt/perl/lib/5.00558/alpha-dec_osf-thread'
+archlibexp='/opt/perl/lib/5.00558/alpha-dec_osf-thread'
 archname64=''
 archname='alpha-dec_osf-thread'
 archobjs=''
@@ -41,6 +41,7 @@ awk='awk'
 baserev='5.0'
 bash=''
 bin='/opt/perl/bin'
+bincompat5005='undef'
 binexp='/opt/perl/bin'
 bison=''
 byacc='byacc'
@@ -50,12 +51,12 @@ castflags='0'
 cat='cat'
 cc='cc'
 cccdlflags=' '
-ccdlflags='  -Wl,-rpath,/opt/perl/lib/5.00557/alpha-dec_osf-thread/CORE'
+ccdlflags='  -Wl,-rpath,/opt/perl/lib/5.00558/alpha-dec_osf-thread/CORE'
 ccflags='-pthread -std -DLANGUAGE_C'
 ccsymbols='__LANGUAGE_C__=1 _LONGLONG=1 LANGUAGE_C=1 SYSTYPE_BSD=1'
 cf_by='jhi'
 cf_email='yourname@yourhost.yourplace.com'
-cf_time='Sun Jul 25 17:08:22 EET DST 1999'
+cf_time='Mon Aug  2 01:32:52 EET DST 1999'
 chgrp=''
 chmod=''
 chown=''
@@ -78,13 +79,28 @@ crosscompile='undef'
 cryptlib=''
 csh='csh'
 d_Gconvert='gcvt((x),(n),(b))'
+d_PRIEldbl='define'
+d_PRIFldbl='define'
+d_PRIGldbl='define'
+d_PRIX64='define'
+d_PRId64='define'
+d_PRIeldbl='define'
+d_PRIfldbl='define'
+d_PRIgldbl='define'
+d_PRIi64='define'
+d_PRIo64='define'
+d_PRIu64='define'
+d_PRIx64='define'
 d_access='define'
 d_accessx='undef'
 d_alarm='define'
 d_archlib='define'
+d_atolf='undef'
+d_atoll='undef'
 d_attribut='undef'
 d_bcmp='define'
 d_bcopy='define'
+d_bincompat5005=''
 d_bsd='undef'
 d_bsdgetpgrp='undef'
 d_bsdsetpgrp='define'
@@ -336,6 +352,7 @@ d_strerror='define'
 d_strtod='define'
 d_strtol='define'
 d_strtoul='define'
+d_strtoull='undef'
 d_strxfrm='define'
 d_suidsafe='undef'
 d_symlink='define'
@@ -483,15 +500,15 @@ i_vfork='undef'
 ignore_versioned_solibs=''
 incpath=''
 inews=''
-installarchlib='/opt/perl/lib/5.00557/alpha-dec_osf-thread'
+installarchlib='/opt/perl/lib/5.00558/alpha-dec_osf-thread'
 installbin='/opt/perl/bin'
 installman1dir='/opt/perl/man/man1'
 installman3dir='/opt/perl/man/man3'
 installprefix='/opt/perl'
 installprefixexp='/opt/perl'
-installprivlib='/opt/perl/lib/5.00557'
+installprivlib='/opt/perl/lib/5.00558'
 installscript='/opt/perl/bin'
-installsitearch='/opt/perl/lib/site_perl/5.00557/alpha-dec_osf-thread'
+installsitearch='/opt/perl/lib/site_perl/5.00558/alpha-dec_osf-thread'
 installsitelib='/opt/perl/lib/site_perl'
 installstyle='lib'
 installusrbinperl='define'
@@ -586,8 +603,8 @@ pmake=''
 pr=''
 prefix='/opt/perl'
 prefixexp='/opt/perl'
-privlib='/opt/perl/lib/5.00557'
-privlibexp='/opt/perl/lib/5.00557'
+privlib='/opt/perl/lib/5.00558'
+privlibexp='/opt/perl/lib/5.00558'
 prototype='define'
 ptrsize='8'
 randbits='48'
@@ -598,6 +615,18 @@ rd_nodata='-1'
 rm='rm'
 rmail=''
 runnm='true'
+sPRIEldbl='"E"'
+sPRIFldbl='"F"'
+sPRIGldbl='"G"'
+sPRIX64='"lX"'
+sPRId64='"ld"'
+sPRIeldbl='"e"'
+sPRIfldbl='"f"'
+sPRIgldbl='"g"'
+sPRIi64='"li"'
+sPRIo64='"lo"'
+sPRIu64='"lu"'
+sPRIx64='"lx"'
 sched_yield='sched_yield()'
 scriptdir='/opt/perl/bin'
 scriptdirexp='/opt/perl/bin'
@@ -619,8 +648,8 @@ sig_name_init='"ZERO", "HUP", "INT", "QUIT", "ILL", "TRAP", "ABRT", "EMT", "FPE"
 sig_num='0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 6 6 16 20 23 23 23 29 48 '
 sig_num_init='0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 6, 6, 16, 20, 23, 23, 23, 29, 48, 0'
 signal_t='void'
-sitearch='/opt/perl/lib/site_perl/5.00557/alpha-dec_osf-thread'
-sitearchexp='/opt/perl/lib/site_perl/5.00557/alpha-dec_osf-thread'
+sitearch='/opt/perl/lib/site_perl/5.00558/alpha-dec_osf-thread'
+sitearchexp='/opt/perl/lib/site_perl/5.00558/alpha-dec_osf-thread'
 sitelib='/opt/perl/lib/site_perl'
 sitelibexp='/opt/perl/lib/site_perl'
 siteprefix='/opt/perl'
@@ -650,7 +679,7 @@ stdio_ptr='((fp)->_ptr)'
 stdio_stream_array='_iob'
 strings='/usr/include/string.h'
 submit=''
-subversion='57'
+subversion='58'
 sysman='/usr/man/man1'
 tail=''
 tar=''
@@ -686,7 +715,7 @@ vendorlib=''
 vendorlibexp=''
 vendorprefix=''
 vendorprefixexp=''
-version='5.00557'
+version='5.00558'
 vi=''
 voidflags='15'
 xlibpth='/usr/lib/386 /lib/386'
@@ -708,7 +737,7 @@ config_arg9='-Dmyhostname=yourhost'
 config_arg10='-dE'
 PERL_REVISION=5
 PERL_VERSION=5
-PERL_SUBVERSION=57
+PERL_SUBVERSION=58
 CONFIGDOTSH=true
 # Variables propagated from previous config.sh file.
 pp_sys_cflags='ccflags="$ccflags -DNO_EFF_ONLY_OK"'
