@@ -58,13 +58,16 @@ $VERSION = "1.03";
 	F_GETFD
 	F_GETFL
 	F_GETLK
+	F_GETLK64
 	F_GETOWN
 	F_POSIX
 	F_RDLCK
 	F_SETFD
 	F_SETFL
 	F_SETLK
+	F_SETLK64
 	F_SETLKW
+	F_SETLKW64
 	F_SETOWN
 	F_SHLCK
 	F_UNLCK
@@ -78,6 +81,7 @@ $VERSION = "1.03";
 	O_DSYNC
 	O_EXCL
 	O_EXLOCK
+	O_LARGEFILE
 	O_NDELAY
 	O_NOCTTY
 	O_NONBLOCK
@@ -118,7 +122,7 @@ sub AUTOLOAD {
     (my $constname = $AUTOLOAD) =~ s/.*:://;
     my $val = constant($constname, 0);
     if ($! != 0) {
-	if ($! =~ /Invalid/) {
+	if ($! =~ /Invalid/ || $!{EINVAL}) {
 	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
 	    goto &AutoLoader::AUTOLOAD;
 	}
