@@ -79,7 +79,11 @@
 #ifdef HAS_SETLINEBUF
 #define PerlIO_setlinebuf(f)		setlinebuf(f);
 #else
-#define PerlIO_setlinebuf(f)		setvbuf(f, Nullch, _IOLBF, 0);
+# ifdef CYGWIN
+#  define PerlIO_setlinebuf(f)
+# else
+#  define PerlIO_setlinebuf(f)		setvbuf(f, Nullch, _IOLBF, 0);
+# endif
 #endif
 
 /* Now our interface to Configure's FILE_xxx macros */
@@ -272,8 +276,14 @@
 #define fputc(c,f)		PerlIO_putc(f,c)
 #define fputs(s,f)		PerlIO_puts(f,s)
 #define getc(f)			PerlIO_getc(f)
+#ifdef getc_unlocked
+#undef getc_unlocked
+#endif
 #define getc_unlocked(f)	PerlIO_getc(f)
 #define putc(c,f)		PerlIO_putc(f,c)
+#ifdef putc_unlocked
+#undef putc_unlocked
+#endif
 #define putc_unlocked(c,f)	PerlIO_putc(c,f)
 #define ungetc(c,f)		PerlIO_ungetc(f,c)
 #if 0
